# from pykson import JsonObject, Pykson, IntegerField, StringField, ObjectListField, ListField, DateField, DateTimeField, TimestampSecondsField, ObjectField, JsonField, \
#     TypeHierarchyAdapter
#
# #
# # class Score(JsonObject):
# #     score = IntegerField()
# #     course = StringField()
# #
# #     def __str__(self):
# #         return str(self.course) + ": " + str(self.score)
# #
# #
# # class Student(JsonObject):
# #
# #     first_name = StringField(serialized_name="fn")
# #     last_name = StringField()
# #     age = IntegerField()
# #     birth = DateField(serialized_name='b')
# #     birth_time = TimestampSecondsField(serialized_name='bt')
# #     # scores = ListField(int)
# #     # scores = ObjectListField(Score)
# #     score = Score()
# #
# #     def __str__(self):
# #         return "first name:" + str(self.first_name) + ", last name: " + str(self.last_name) + ", birth: " + str(self.birth_time) + ", age: " + str(self.age) + ", score: " + str(self.score)
# #
# #
# # # print(JsonObject.get_fields(Student))
# #
# #
# # # json_text = '{"fn":"ali", "last_name":"soltani", "age": 25, "scores": [ 20, 19]}'
# # # json_text = '{"fn":"ali", "last_name":"soltani", "b":"2015-10-21", "bt": 1553717064, "age": 25, "scores": [ {"course": "algebra", "score": 20}, {"course": "statistics", "score": 19} ]}'
# # json_text = '{"fn":"ali", "last_name":"soltani", "b":"2015-10-21", "bt": 1553717064, "age": 25, "score": {"course": "algebra", "score": 20}}'
# # item = Pykson.from_json(json_text, Student)
# #
# # print(item)
# # print(type(item))
# #
# # print(Pykson.to_json(item))
#
#
# # print(type('ss') == str)
#
#
#
#
# from pykson import Pykson, JsonObject, IntegerField, StringField, ObjectField
#
#
# class Score(JsonObject):
#     score = IntegerField(serialized_name='s')
#     course = StringField(serialized_name='c')
#     data = JsonField(serialized_name='d')
#
#
# class School(JsonObject):
#     name = StringField(serialized_name='n')
#
#
# class Student(JsonObject):
#
#     first_name = StringField(serialized_name="fn")
#     last_name = StringField(serialized_name="ln")
#     age = IntegerField(serialized_name="a", default_value=25, null=False)
#     # school = ObjectField(School, serialized_name="sc")
#     # scores = ObjectListField(Score, serialized_name="s")
#
#
# st = Student(first_name='St1', last_name='st2', age=20)
# print(Pykson().to_json(st))
#
# stp = Pykson().from_json({"fn": "St1", "ln": "st2"}, Student)
# print(stp.age)
# print(Pykson().to_json(stp))
#
#
# # # json_text = '{"fn":"John", "ln":"Smith", "a": 25, "sc": {"n": "MIT"}, "s": [{"s": 100, "c":"Algebra", "d": {"m" :"Metod"}}]}'
# # # student = Pykson().from_json(json_text, Student)
# # # print(Pykson().to_json(student))
# # # a = student.scores[0]
# # # student.school
# #
# #
# # class Course(JsonObject):
# #     name = StringField(serialized_name='n')
# #
# #
# # class MathCourse(Course):
# #     teacher_name = StringField(serialized_name='tn')
# #
# #     def __str__(self):
# #         return str(self.name) + ' ' + str(self.teacher_name)
# #
# #     def __repr__(self):
# #         return self.__str__()
# #
# #
# # class FootballCourse(Course):
# #     coach_name = StringField(serialized_name='cn')
# #
# #     def __str__(self):
# #         return str(self.name) + ' ' + str(self.coach_name)
# #
# #     def __repr__(self):
# #         return self.__str__()
# #
# #
# # class Semester(JsonObject):
# #     course1 = ObjectField(Course, serialized_name="c1")
# #     course2 = ObjectField(Course, serialized_name="c2")
# #
# #     def __str__(self):
# #         return '{c1:' + str(self.course1) + ', c2:' + str(self.course2)+'}'
# #
# #     def __repr__(self):
# #         return self.__str__()
# #
# # pson = Pykson()
# #
# # class J0(JsonObject):
# #     a = StringField(serialized_name='a')
# #
# #
# # class J1(J0):
# #     b = StringField(serialized_name='b')
# #
# #
# # class J2(J1):
# #     c = StringField(serialized_name='c')
# #
# # aa = J2(a='1', b='2', c='3')
# #
# # print(aa.a)
# #
# # jj = pson.to_json(aa)
# # print(jj)
# #
# # bb = pson.from_json('{"c": "3", "b": "2", "a": "4"}', J2)
# #
# # print(bb.a)
# # print(pson.to_json(bb))
#
# # from enum import Enum
# #
# # from pykson import EnumIntegerField
# #
# # class MyEnum(Enum):
# #     A = 1
# #     B = 2
# #
# #
# # class MyJson(JsonObject):
# #     a = ListField(serialized_name='aaa', item_type=EnumIntegerField(null=False, enum=MyEnum))
# #     b = ListField(serialized_name='bbb', item_type=str)
# #
# #
# # mj = MyJson(a=[MyEnum.A, MyEnum.B], b=['1', '3'])
# #
# # pson = Pykson()
# # mjj = pson.to_json(mj)
# #
# # print(mjj)
# #
# # print(pson.to_json(pson.from_json(mjj, MyJson)))
#
#
# # pson = Pykson()
# # pson.register_type_hierarchy_adapter(TypeHierarchyAdapter(
# #     Course,
# #     't',
# #     {
# #         'm': MathCourse,
# #         'f': FootballCourse
# #     }
# # ))
# #
# # semester = [Semester(course1=MathCourse(teacher_name='tavakoli', name='math'),
# #                     course2=FootballCourse(coach_name='ghomali', name='football'))]
# # semester_json = pson.to_json(semester)
# # print(semester_json)
# # d_semester = pson.from_json(semester_json, Semester)
# # print(d_semester)
# #
# #
# # m_course = [MathCourse(teacher_name='tavakoli', name='math'), FootballCourse(coach_name='ghomali', name='football')]
# # m_json = pson.to_json(m_course)
# # print(m_json)
# # d_m_course = pson.from_json(m_json, Course)
# # print(d_m_course)
# # print(type(d_m_course))
#
#
# # class NormalizedOHLCAggregatedIntradayTrade(pykson.JsonObject):
# #     SIDE_NONE = 0
# #     SIDE_BUY = 1
# #     SIDE_SELL = 2
# #
# #     time_point = pykson.TimestampMillisecondsField(serialized_name='tp')
# #     open_price = pykson.FloatField(serialized_name='o')
# #     high_price = pykson.FloatField(serialized_name='h')
# #     low_price = pykson.FloatField(serialized_name='l')
# #     close_price = pykson.FloatField(serialized_name='c')
# #     volume = pykson.FloatField(serialized_name='v')
# #     trade_count = pykson.IntegerField(serialized_name='tc')
# #     sell_trade_volume = pykson.FloatField(serialized_name='stv')
# #     sell_trade_count = pykson.IntegerField(serialized_name='stc')
# #     buy_trade_volume = pykson.FloatField(serialized_name='btv')
# #     buy_trade_count = pykson.IntegerField(serialized_name='btc')
# #
# #
# # class NormalizedOrderBookRow(pykson.JsonObject):
# #     ask_count = pykson.IntegerField(serialized_name='ac')
# #     ask_price = pykson.FloatField(serialized_name='ap')
# #     ask_volume = pykson.FloatField(serialized_name='av')
# #     bid_volume = pykson.FloatField(serialized_name='bv')
# #     bid_price = pykson.FloatField(serialized_name='bp')
# #     bid_count = pykson.IntegerField(serialized_name='bc')
# #
# #
# # class NormalizedOrderBook(pykson.JsonObject):
# #     row1 = pykson.ObjectField(NormalizedOrderBookRow, serialized_name='r1')
# #     row2 = pykson.ObjectField(NormalizedOrderBookRow, serialized_name='r2')
# #     row3 = pykson.ObjectField(NormalizedOrderBookRow, serialized_name='r3')
# #     row4 = pykson.ObjectField(NormalizedOrderBookRow, serialized_name='r4')
# #     row5 = pykson.ObjectField(NormalizedOrderBookRow, serialized_name='r5')
# #
# #
# # class NormalizedIntradayAggregatedData(pykson.JsonObject):
# #     time_point = pykson.TimestampMillisecondsField(serialized_name='tp')
# #     ohlcv_trade = pykson.ObjectField(NormalizedOHLCAggregatedIntradayTrade, serialized_name='t')
# #     order_book = pykson.ObjectField(NormalizedOrderBook, serialized_name='b')
# #
# #
# # class NormalizedDayAggregated(pykson.JsonObject):
# #     day = pykson.DateField(serialized_name='d')
# #     aggregated_data = pykson.ObjectListField(NormalizedIntradayAggregatedData, serialized_name='ad')
# #
# # import requests
# # resp = requests.get('http://127.0.0.1:8000/crawler/intraday/IRO1BMLT0001/from/2019-03-16/to/2019-03-19/data_points_normalized/5m/')
# # resp_data = Pykson.from_json(resp.text, NormalizedDayAggregated)
# # print(resp_data)

from pykson.generator import PyksonGenerator

book_json = {
    'name': 'Harry Potter and the Deathly Hallows',
    'pages': 607,
    'publication_date': '27 July 2007',
    'author': {
        'firstName': 'J.K.',
        'lastName': 'Rowling',
        'age': 57,
        'dateOfBirth': '1965-06-31 ',
    },
    'reviews': [
        {
            'publisher': 'The New York Times',
            'reviewerName': 'Stephen King',
            'reviewDate': 'July 23, 2000',
            'rate': 10.0,
        },
        {
            'publisher': 'Kirkus Reviews',
            'reviewerName': 'Kirkus Reviews',
            'reviewDate': 'JULY 8, 2000',
            'rate': 9.5
        },
    ]
}
PyksonGenerator.generate_pykson_classes(book_json, 'Book', generate_test=True)



import re
import _io
import json
from dateutil import parser
from typing import List, Optional, Tuple, Dict, Set, Any


class PyksonGenerator:
    _initial_letter_replacements = {
        '0': 'zero',
        '1': 'one',
        '2': 'two',
        '3': 'three',
        '4': 'four',
        '5': 'five',
        '6': 'six',
        '7': 'seven',
        '8': 'eight',
        '9': 'nine.',
        '.': 'dot',
        ':': 'colon',
        ';': 'semicolon',
        '"': 'quote',
        '#': 'sharp',
        '%': 'percent',
        '!': 'danger',
        '?': 'question',
        '&': 'and',
        '*': 'star',
    }
    _reserved_words = [
        'from', 'class', 'for', 'if', 'in', 'def', 'return', 'break', 'continue', 'pass', 'id', 'else', 'elif',
        'object',
    ]

    class ValidationError(Exception):
        pass

    @staticmethod
    def _null_or_equal_type_strings(a: str, b: str):
        return a == 'NoneType' or b == 'NoneType' or a == b

    @staticmethod
    def _null_or_equal(a: str, b: str):
        return a is None or b is None or a == b

    class SchemaProperty:
        def __init__(
                self,
                name: str,
                type_str: str,
                item_type_str: Optional[str] = None,
                nullable: bool = False,
                values: Optional[Set[Any]] = None
        ):
            self.name = name
            self.type_str = type_str
            self.item_type_str = item_type_str
            self.nullable = nullable
            self.values = values

        def json_repr(self):
            js = {'name': self.name, 'type': self.type_str, 'null': self.nullable}
            if self.item_type_str:
                js['item_type'] = self.item_type_str
            if self.values:
                js['values'] = list([str(v) for v in self.values])
            return js

        def __str__(self):
            json_obj = self.json_repr()
            return json.dumps(json_obj, indent=2)

    class Schema:
        def __init__(
                self,
                name: str,
                properties: List['PyksonGenerator.SchemaProperty'],
        ):
            self.name = name
            self.properties = properties

        def json_repr(self):
            return {'name': self.name, 'properties': [p.json_repr() for p in self.properties]}

        def __str__(self):
            json_obj = self.json_repr()
            return json.dumps(json_obj, indent=2)

        def get_properties_by_name(self) -> Dict[str, 'PyksonGenerator.SchemaProperty']:
            return {
                p.name: p for p in self.properties
            }

        def is_similar(self, other_schema: 'PyksonGenerator.Schema'):
            prop_by_name = self.get_properties_by_name()
            other_prop_by_name = other_schema.get_properties_by_name()
            return prop_by_name.keys() == other_prop_by_name.keys() and all([
                PyksonGenerator._null_or_equal_type_strings(prop_by_name[n].type_str, other_prop_by_name[n].type_str)
                and
                (
                    True if prop_by_name[n].type_str != list.__name__
                    else PyksonGenerator._null_or_equal(
                        prop_by_name[n].item_type_str, other_prop_by_name[n].item_type_str
                    )
                )
                for n in prop_by_name.keys()
            ])

        def update_from(self, other_schema: 'PyksonGenerator.Schema'):
            assert self.is_similar(other_schema), 'Cannot update from non-similar schema'
            properties = self.properties
            other_props = other_schema.get_properties_by_name()
            for prop in properties:
                other_prop: PyksonGenerator.SchemaProperty = other_props[prop.name]
                if prop.type_str == 'NoneType' and other_prop.type_str != 'NoneType':
                    prop.type_str = other_prop.type_str
                if prop.type_str == list.__name__ and other_prop.type_str == list.__name__:
                    if prop.item_type_str is None and other_prop.item_type_str is not None:
                        prop.item_type_str = other_prop.item_type_str
                else:
                    if other_prop.values is not None and len(other_prop.values) > 0:
                        if prop.values is None:
                            prop.values = set()
                        prop.values.update(other_prop.values)
            self.properties = properties

    @staticmethod
    def _is_primitive(element) -> bool:
        if any([isinstance(element, cls) for cls in [int, str, float, bool, bytes]]):
            return True
        else:
            return False

    @staticmethod
    def _is_list_of_primitives(element) -> bool:
        if not isinstance(element, list):
            return False
        return all([
            any([isinstance(list_item, cls) for cls in [int, str, float, bool, bytes]])
            for list_item in element
        ])

    @staticmethod
    def _is_list_of_similar_items(element) -> bool:
        if not isinstance(element, list):
            return False
        return len(set([type(list_item) for list_item in element])) <= 1

    @staticmethod
    def _add_schema_to_list_if_does_not_exist(
            schema_list: List['PyksonGenerator.Schema'], new_schema: 'PyksonGenerator.Schema'
    ) -> Optional['PyksonGenerator.Schema']:
        found_new_schema: bool = False
        duplicate_schema: Optional[PyksonGenerator.Schema] = None
        for s in schema_list:
            if new_schema.is_similar(s):
                found_new_schema = True
                s.update_from(new_schema)
                duplicate_schema = s
                break
        if found_new_schema is False:
            schema_list.append(new_schema)
        return duplicate_schema

    @staticmethod
    def generate_schema(
            json_object: dict,
            name: str,
            sub_schemas: Optional[List['PyksonGenerator.Schema']] = None
    ) -> Tuple['PyksonGenerator.Schema', List['PyksonGenerator.Schema']]:
        if sub_schemas is None:
            sub_schemas: List[PyksonGenerator.Schema] = []
        properties: List[PyksonGenerator.SchemaProperty] = []
        for key, value in json_object.items():
            if PyksonGenerator._is_primitive(value):
                properties.append(PyksonGenerator.SchemaProperty(
                    name=key, type_str=type(value).__name__, values={value}
                ))
            elif value is None:
                properties.append(
                    PyksonGenerator.SchemaProperty(
                        name=key, type_str=type(value).__name__, nullable=True
                    )
                )
            elif isinstance(value, list):
                pass
                if not PyksonGenerator._is_list_of_similar_items(value):
                    raise PyksonGenerator.ValidationError(
                        f'Cannot generate schema for list of non-similar items {[type(t) for t in value]}'
                    )
                else:
                    if len(value) == 0:
                        properties.append(PyksonGenerator.SchemaProperty(
                            name=key, type_str=list.__name__, item_type_str=type(None).__name__
                        ))
                    elif PyksonGenerator._is_list_of_primitives(value):
                        properties.append(
                            PyksonGenerator.SchemaProperty(
                                name=key, type_str=list.__name__, item_type_str=type(value[0]).__name__
                            )
                        )
                    else:
                        assert all([isinstance(v, dict) for v in value]), f'List with unknown type {type(value[0])} ' \
                                                                          f'found {value}'
                        list_item_name = f'{name}.{key}'
                        value_0_schema, value_0_sub_schemas = PyksonGenerator.generate_schema(
                            value[0], list_item_name, sub_schemas
                        )
                        dup_schema = PyksonGenerator._add_schema_to_list_if_does_not_exist(sub_schemas, value_0_schema)
                        value_0_schema_name = (dup_schema or value_0_schema).name

                        for i in range(1, len(value)):
                            value_i_schema, value_i_sub_schemas = PyksonGenerator.generate_schema(
                                value[i], list_item_name, sub_schemas
                            )
                            if not value_0_schema.is_similar(value_i_schema):
                                raise Exception(f'Non similar list schemas {value_0_schema} AND {value_i_schema} '
                                                f'in list {list_item_name}')
                            else:
                                value_0_schema.update_from(value_i_schema)
                        properties.append(PyksonGenerator.SchemaProperty(
                            name=key, type_str=list.__name__, item_type_str=value_0_schema_name
                        ))

            elif isinstance(value, dict):
                value_schema, value_sub_schemas = PyksonGenerator.generate_schema(value, f'{name}.{key}', sub_schemas)
                duplicate_schema = PyksonGenerator._add_schema_to_list_if_does_not_exist(sub_schemas, value_schema)
                value_schema_name = (duplicate_schema or value_schema).name
                properties.append(PyksonGenerator.SchemaProperty(
                    name=key, type_str=value_schema_name
                ))
            else:
                raise Exception(f"Unsupported value {value} of type {type(value)} in {name}")

        for i in range(0, len(sub_schemas)):
            for j in range(i + 1, len(sub_schemas)):
                if sub_schemas[i].is_similar(sub_schemas[j]):
                    raise Exception(
                        f'Found duplicate schemas in final sub schemas {sub_schemas[i]} AND {sub_schemas[j]}'
                    )
        return PyksonGenerator.Schema(name=name, properties=properties), sub_schemas

    @staticmethod
    def _schema_name_to_class_name(schema_name: str) -> str:
        for i in PyksonGenerator._initial_letter_replacements.keys():
            if schema_name.startswith(i):
                schema_name = PyksonGenerator._initial_letter_replacements[i] + '.' + schema_name[1:]
                break
        components = schema_name.split('.')
        return ''.join(x[0].upper() + x[1:] for x in components)

    @staticmethod
    def _to_snake_case(text: str) -> str:
        text = re.sub(r'(?<!^)(?=[A-Z])', '_', text).lower()
        for i in PyksonGenerator._initial_letter_replacements.keys():
            if text.startswith(i):
                text = PyksonGenerator._initial_letter_replacements[i] + '_' + text[1:]
                break
        for w in PyksonGenerator._reserved_words:
            if text == w:
                text = text + '_'
                break
        return text

    @staticmethod
    def _is_datetime(value: str):
        if value is None:
            return True
        if len(value) < 8:
            return False
        # noinspection PyBroadException
        try:
            parser.parse(value)
            return True
        except Exception:
            return False

    @staticmethod
    def _are_all_datetime(values: Set[str]) -> bool:
        if values is None:
            return False
        if len(values) == 0:
            return False
        if all([v is None for v in values]):
            return False
        return all([
            PyksonGenerator._is_datetime(v) for v in values
        ])

    @staticmethod
    def write_pykson_class(
            schema: 'PyksonGenerator.Schema',
            file_writer: _io.TextIOWrapper,
            indent: str,
            include_todos: bool
    ):
        file_writer.write(f'class {PyksonGenerator._schema_name_to_class_name(schema.name)}(pykson.JsonObject):\n')
        for p in schema.properties:
            primitive_field_to_pykson_class = {
                int.__name__: 'IntegerField',
                # we exclude string to check for datetimes
                # str.__name__: 'StringField',
                float.__name__: 'FloatField',
                bool.__name__: 'BooleanField',
                bytes.__name__: 'BytesField',
            }
            primitive_field_to_pykson_field_type = {
                int.__name__: 'int',
                str.__name__: 'str',
                float.__name__: 'float',
                bool.__name__: 'bool',
                bytes.__name__: 'bytes',
            }
            if p.type_str in primitive_field_to_pykson_class.keys():
                file_writer.write(
                    f'{indent}{PyksonGenerator._to_snake_case(p.name)} = '
                    f'pykson.{primitive_field_to_pykson_class[p.type_str]}'
                    f'(serialized_name="{p.name}", null={p.nullable})'
                )
            elif p.type_str == str.__name__:
                if p.values is not None and len(p.values) > 0 and PyksonGenerator._are_all_datetime(p.values):
                    file_writer.write(
                        f'{indent}{PyksonGenerator._to_snake_case(p.name)} = '
                        f'pykson.DateTimeField(datetime_format=None, serialized_name="{p.name}", null={p.nullable})'
                    )
                else:
                    file_writer.write(
                        f'{indent}{PyksonGenerator._to_snake_case(p.name)} = '
                        f'pykson.StringField(serialized_name="{p.name}", null={p.nullable})'
                    )
            elif p.type_str == 'NoneType':
                if include_todos:
                    file_writer.write(f'{indent}# todo: Fix this line, value was always None!\n')
                # file_writer.write(f'{indent}{_to_snake_case(p.name)} = None')
                file_writer.write(f'{indent}{PyksonGenerator._to_snake_case(p.name)} = pykson.StringField'
                                  f'(serialized_name="{p.name}", null=True)')
            # elif p.type_str == datetime.datetime.__name__:
            #     file_writer.write(f'{indent}{PyksonGenerator._to_snake_case(p.name)} = pykson.StringField'
            #                       f'(serialized_name="{p.name}", null=True)')
            elif p.type_str == list.__name__:
                if p.item_type_str is None:
                    if include_todos:
                        file_writer.write(
                            f'{indent}# todo: Fix this line, list was empty and cannot detect item type!\n')
                    file_writer.write(f'{indent}{PyksonGenerator._to_snake_case(p.name)} = pykson.ListField'
                                      f'(item_type=Any, serialized_name="{p.name}", null={p.nullable})')
                elif p.item_type_str in primitive_field_to_pykson_field_type.keys():
                    file_writer.write(f'{indent}{PyksonGenerator._to_snake_case(p.name)} = pykson.ListField'
                                      f'(item_type={primitive_field_to_pykson_field_type[p.item_type_str]}, '
                                      f'serialized_name="{p.name}", null={p.nullable})')
                else:
                    file_writer.write(f'{indent}{PyksonGenerator._to_snake_case(p.name)} = pykson.ObjectListField'
                                      f'(item_type={PyksonGenerator._schema_name_to_class_name(p.item_type_str)}, '
                                      f'serialized_name="{p.name}", null={p.nullable})')
            else:
                file_writer.write(f'{indent}{PyksonGenerator._to_snake_case(p.name)} = pykson.ObjectField('
                                  f'item_type={PyksonGenerator._schema_name_to_class_name(p.type_str)}, '
                                  f'serialized_name="{p.name}", null={p.nullable})')
            file_writer.write('\n')

    # noinspection PyTypeChecker
    @staticmethod
    def generate_pykson_classes(
            json_object: dict,
            base_name: str,
            indent: str = '    ',
            generate_test: bool = False,
            include_todos: bool = True,
    ):
        schema, sub_schemas = PyksonGenerator.generate_schema(json_object, name=base_name)
        with open(f'pykson_{PyksonGenerator._to_snake_case(base_name)}.generated.py', 'w') as f:
            f.write('import pykson\n')
            f.write('\n\n')
            for s_c in sub_schemas:
                PyksonGenerator.write_pykson_class(s_c, f, indent, include_todos=include_todos)
                f.write('\n')
                f.write('\n')
            PyksonGenerator.write_pykson_class(schema, f, indent, include_todos=include_todos)
            if generate_test is True:
                f.write('\n')
                f.write('\n')
                f.write('if __name__ == "__main__":\n')
                dumped_string = json.dumps(json_object, indent=indent).replace('\n', f'\n{indent}')
                dumped_string = dumped_string.replace(': null\n', ': None\n')
                dumped_string = dumped_string.replace(': false\n', ': False\n')
                dumped_string = dumped_string.replace(': true\n', ': True\n')
                dumped_string = dumped_string.replace(': null,\n', ': None,\n')
                dumped_string = dumped_string.replace(': false,\n', ': False,\n')
                dumped_string = dumped_string.replace(': true,\n', ': True,\n')
                f.write(indent + 'test_json = ' + dumped_string)
                f.write(f'{indent}\n')
                f.write(f'{indent}obj = pykson.Pykson().from_json(test_json, {schema.name})')
                f.write('\n')



import pykson


class BookAuthor(pykson.JsonObject):
    first_name = pykson.StringField(serialized_name="firstName", null=False)
    last_name = pykson.StringField(serialized_name="lastName", null=False)
    age = pykson.IntegerField(serialized_name="age", null=False)
    date_of_birth = pykson.StringField(serialized_name="dateOfBirth", null=False)


class BookReviews(pykson.JsonObject):
    publisher = pykson.StringField(serialized_name="publisher", null=False)
    reviewer_name = pykson.StringField(serialized_name="reviewerName", null=False)
    review_date = pykson.DateTimeField(datetime_format=None, serialized_name="reviewDate", null=False)
    rate = pykson.FloatField(serialized_name="rate", null=False)


class Book(pykson.JsonObject):
    name = pykson.StringField(serialized_name="name", null=False)
    pages = pykson.IntegerField(serialized_name="pages", null=False)
    publication_date = pykson.DateTimeField(datetime_format=None, serialized_name="publication_date", null=False)
    author = pykson.ObjectField(item_type=BookAuthor, serialized_name="author", null=False)
    reviews = pykson.ObjectListField(item_type=BookReviews, serialized_name="reviews", null=False)


if __name__ == "__main__":
    test_json = {
        "name": "Harry Potter and the Deathly Hallows",
        "pages": 607,
        "publication_date": "27 July 2007",
        "author": {
            "firstName": "J.K.",
            "lastName": "Rowling",
            "age": 57,
            "dateOfBirth": "1965-06-31 "
        },
        "reviews": [
            {
                "publisher": "The New York Times",
                "reviewerName": "Stephen King",
                "reviewDate": "July 23, 2000",
                "rate": 10.0
            },
            {
                "publisher": "Kirkus Reviews",
                "reviewerName": "Kirkus Reviews",
                "reviewDate": "JULY 8, 2000",
                "rate": 9.5
            }
        ]
    }    
    obj = pykson.Pykson().from_json(test_json, Book)
